# -*- coding: utf-8 -*-
from django.core.urlresolvers import resolve
from django.test import TestCase

from sure import expect

from socialnetworks.facebook import views


class FacebookUrlsTestCase(TestCase):
    def test_login_url_is_correct(self):
        url = resolve('/social/facebook/login/')

        expect(url) \
            .to.have.property('func') \
            .to.have.property('__name__') \
            .being.equal(views.FacebookDialogRedirectView.__name__)

    def test_callback_url_is_correct(self):
        url = resolve('/social/facebook/callback/')

        expect(url) \
            .to.have.property('func') \
            .to.have.property('__name__') \
            .being.equal(views.FacebookCallbackView.__name__)

    def test_setup_url_is_correct(self):
        url = resolve('/social/facebook/setup/')

        expect(url) \
            .to.have.property('func') \
            .to.have.property('__name__') \
            .being.equal(views.FacebookSetupView.__name__)

    def test_disconnect_url_is_correct(self):
        url = resolve('/social/facebook/disconnect/')

        expect(url) \
            .to.have.property('func') \
            .to.have.property('__name__') \
            .being.equal(views.FacebookOAuthDisconnectView.__name__)



import pytz

from datetime import datetime
from random import randint

from django.contrib.auth import get_user_model
from django.core import signing

from unidecode import unidecode

from .settings import COOKIE_MAX_AGE


def read_social_data(request, key):
    """
    Search the current request for user's social information, search first
    in the user's session, if there is no information then search in the
    request's cookies.

    Returns a dictionary of the social information if found, otherwise
    returns None.
    """
    data = request.session.get(key, request.get_signed_cookie(
        key, max_age=COOKIE_MAX_AGE, default=None))

    return signing.loads(data) if data else None


def compose_username(data):
    """
    Returns a suggested username for a new user.

    If username is passed as a key in the given data dictionary it will be
    used as a base for the suggestion, otherwise the base will be generated by
    the concatenation of the fist and last name of the given data.
    Then the base will be unidecoded and space cleaned and then it will be
    concatenated with a random number between 1 and 9999.
    """
    UserModel = get_user_model()

    if 'username' in data and data['username']:
        raw_name = data['username']

    else:
        raw_name = data['first_name'] + data['last_name']

    unidecoded_name = unidecode(raw_name.replace(' ', ''))
    randomize = lambda n: '%s%d' % (n, randint(1, 9999))
    valid = False

    while not valid:
        username = randomize(unidecoded_name)

        if not UserModel.objects.filter(username__iexact=username):
            valid = True

    return username


def to_timestamp(date_time):
    """
    Transform a Python datetime object to a UNIX UTC timestamp.
    """
    if date_time is None:
        return None

    utc_dt = date_time.replace(tzinfo=pytz.UTC)
    delta = utc_dt - datetime(1970, 1, 1, tzinfo=pytz.UTC)

    return delta.total_seconds()


def from_timestamp(timestamp):
    """
    Transform a UNIX UTC timestamp to a Python datetime object.
    """
    if timestamp is None:
        return None

    return datetime.fromtimestamp(timestamp, tz=pytz.UTC)



from datetime import timedelta

from django.utils.timezone import now

from . import settings
from .models import MovesAppOAuthProfile
from ..core.clients import OAuth2Client


class MovesAppClient(OAuth2Client):
    """
    Client to connect to the Moves app API.
    """
    service_name = 'Moves app'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    scope = settings.SCOPE
    model = MovesAppOAuthProfile

    expiration_label = 'expires_in'
    authorization_url = 'https://api.moves-app.com/oauth/v1/authorize'
    access_token_url = 'https://api.moves-app.com/oauth/v1/access_token'
    token_debug_url = 'https://api.moves-app.com/oauth/v1/tokeninfo'
    refresh_token_url = 'https://api.moves-app.com/oauth/v1/access_token'
    service_api_url = 'https://api.moves-app.com/api/1.1/'
    session_key = 'socialnetworks:moves-app'

    def compose_authorization_url(self, callback_url):
        """
        Return the url to request user authorization at Moves app.
        """
        params = {
            'response_type': 'code',
            'client_id': self.app_key,
            'redirect_uri': callback_url,
            'scope': self.scope,
        }

        return self.encode_url(self.authorization_url, params)

    def debug_access_token(self, token=None):
        """
        Check if the given access token is yet valid.

        Return a tuple of two objects where the first element is a boolean
        that tells whether the token is valid or not and the second element is
        the data resulting of the token validation.
        """
        if token is None:
            token = self._oauth_data['access_token']

        params = {'access_token': token}

        response = self._get(self.token_debug_url, params=params)

        return (response.status_code == 200, response.json())

    def refresh_access_token(self, update=True):
        """
        Perform a request to the Moves app API to refresh the access token for
        the given profile.

        Return a tuple of two objects where the first element is a boolean
        telling whether the refresh request was successfull and the secons
        element is the data returned by the Moves app API.

        To call this method, client instances must be initialized with a
        proper ```MovesAppOAuthProfile``` instance.

        Pass update=True to force the profile instance to be updated with the
        retrieved data.
        """
        assert isinstance(self._profile, self.model)

        params = {
            'grant_type': 'refresh_token',
            'refresh_token': self._profile.oauth_refresh_token,
            'client_id': self.app_key,
            'client_secret': self.app_secret
        }

        response = self._post(self.refresh_token_url, params=params)
        response_data = response.json()

        if update and response.status_code == 200:
            self._profile.oauth_access_token = response_data.get(
                self.access_token_label)

            self._profile.oauth_refresh_token = response_data.get(
                self.refresh_token_label)

            self._profile.oauth_token_expires_at = now() + timedelta(
                seconds=int(response_data.get(self.expiration_label)))

            self._profile.save()

        return (response.status_code == 200, response.json())



from .clients import LinkedInClient
from .settings import SETUP_URL_NAME
from ..core import views


class LinkedInDialogRedirect(views.OAuthDialogRedirectView):
    """
    View that handles the redirects for the LinkedIn authorization dialog.
    """
    client_class = LinkedInClient


class LinkedInCallback(views.OAuthCallbackView):
    """
    View that handles the LinkedIn OAuth callback.
    """
    client_class = LinkedInClient


class LinkedInSetup(views.OAuthSetupView):
    """
    View that handles the setup of a retrieved LinkedIn account.
    """
    client_class = LinkedInClient
    setup_url = SETUP_URL_NAME


class LinkedInOAuthDisconnect(views.OAuthDisconnectView):
    """
    View that handles the disconnect of a LinkedIn account.
    """
    client_class = LinkedInClient



# -*- coding: utf-8 -*-
from django.core.urlresolvers import resolve
from django.test import TestCase

from sure import expect

from socialnetworks.twitter import views


class TwitterUrlsTestCase(TestCase):
    def test_login_url_is_correct(self):
        url = resolve('/social/twitter/login/')

        expect(url) \
            .to.have.property('func') \
            .to.have.property('__name__') \
            .being.equal(views.TwitterDialogRedirectView.__name__)

    def test_callback_url_is_correct(self):
        url = resolve('/social/twitter/callback/')

        expect(url) \
            .to.have.property('func') \
            .to.have.property('__name__') \
            .being.equal(views.TwitterCallbackView.__name__)

    def test_setup_url_is_correct(self):
        url = resolve('/social/twitter/setup/')

        expect(url) \
            .to.have.property('func') \
            .to.have.property('__name__') \
            .being.equal(views.TwitterSetupView.__name__)

    def test_disconnect_url_is_correct(self):
        url = resolve('/social/twitter/disconnect/')

        expect(url) \
            .to.have.property('func') \
            .to.have.property('__name__') \
            .being.equal(views.TwitterOAuthDisconnectView.__name__)



from uuid import uuid4

from . import settings
from ..core.clients import OAuth2Client
from .models import PayPalOAuthProfile


class PayPalClient(OAuth2Client):
    """
    Client to connect with the PayPal REST API.
    """
    # Configuration
    service_name = 'PayPal'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    app_access_token = settings.APP_ACCESS_TOKEN
    scope = settings.SCOPE
    model = PayPalOAuthProfile

    expiration_label = 'expires_in'

    is_live = settings.IS_LIVE
    token_debug_url = 'identity/openidconnect/userinfo/?schema=openid'
    session_key = 'socialnetworks:paypal'

    def get_base_api_domain(self):
        """
        Return the base paypal domain depending in the 'IS_LIVE' setting.
        """
        return 'api.paypal.com' if self.is_live else 'api.sandbox.paypal.com'

    @property
    def authorization_url(self):
        """
        Return the proper authorization url depending in the 'IS_LIVE' setting.
        """
        if self.is_live:
            return ('https://www.paypal.com/webapps/auth/protocol/'
                    'openidconnect/v1/authorize')
        else:
            return ('https://www.sandbox.paypal.com/webapps/auth/protocol/'
                    'openidconnect/v1/authorize')

    @property
    def access_token_url(self):
        """
        Return the proper aacces token url depending in the 'IS_LIVE' setting.
        """
        return ('https://%s/v1/identity/openidconnect/tokenservice' %
                self.get_base_api_domain())

    @property
    def service_api_url(self):
        """
        Return the proper api url depending in the 'IS_LIVE' setting.
        """
        return 'https://%s/v1/' % self.get_base_api_domain()

    def compose_authorization_url(self, callback_url):
        """
        Return the url to request user authorization at PayPal.
        """
        params = {
            'client_id': self.app_key,
            'redirect_uri': callback_url,
            'response_type': 'code',
            'scope': self.scope,
            'nonce': uuid4()
        }

        return self.encode_url(self.authorization_url, params)

    def debug_access_token(self, token=None):
        """
        Check if the given access token is yet valid.

        Return a tuple of two objects where the first element is a boolean
        that tells whether the token is valid or not and the second element is
        the data resulting of the token validation.
        """
        if token is None:
            token = self._oauth_data['access_token']

        auth = self.compose_auth({'access_token': token})
        r = self.get(self.token_debug_url, auth=auth)

        return ('user_id' in r, r)

    def retrieve_user_data(self):
        """
        Return the available data of the user from PayPal.
        """
        r = self.get(self.token_debug_url)

        return {
            'first_name': r['given_name'],
            'last_name': r['family_name'],
            'email': r['email']
        }



from .clients import MovesAppClient
from .settings import SETUP_URL_NAME
from ..core import views


class MovesAppDialogRedirect(views.OAuthDialogRedirectView):
    """
    View that handles the redirects for the Moves app authorization dialog.
    """
    client_class = MovesAppClient


class MovesAppCallback(views.OAuthCallbackView):
    """
    View that handles the Moves app OAuth callback.
    """
    client_class = MovesAppClient


class MovesAppSetup(views.OAuthSetupView):
    """
    View that handles the setup of a retrieved Moves app account.
    """
    client_class = MovesAppClient
    setup_url = SETUP_URL_NAME


class MovesAppOAuthDisconnect(views.OAuthDisconnectView):
    """
    View that handles the disconnect of a Moves app account.
    """
    client_class = MovesAppClient



import requests

from django import template
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from ...core.defaults import (
    DEFAULT_ERROR_MESSAGE, DEFAULT_LOGIN_LABEL, DEFAULT_SIGNIN_LABEL)


register = template.Library()


@register.simple_tag(takes_context=True)
def twitter_login(context, label=None, css_class=None, icon_class=None,
                  only_login=False, error_message=None, error_class=None):
    """
    Renders a 'Sign in with Twitter' button.

    Parameters:
        - label: text to show as button label.
        - css_class: css class to apply to the button.
        - icon_class: css class to apply to the empty <i> tag placed inside
            (left) of the button, useful if used with css icons like
            "Font Awesome" or "Glyphicons".
        - only_login: if True the button will only try to log the user in
            without registering it, if an user that matches the Twitter
            credentials is not found then the user will be redirected to the
            page where the button was originally displayed and an a error
            message will be shown.
        - error_message: text to show as login error, if applicable.
        - error_class: css class to render the error message, if applicable.
    """
    replacements = {
        'service': 'Twitter'
    }

    context['action'] = reverse('socialnetworks:twitter:login')
    context['css_class'] = css_class
    context['icon_class'] = icon_class
    context['error_class'] = error_class
    context['only_login'] = only_login

    context['label'] = (
        label or
        (DEFAULT_LOGIN_LABEL if only_login else DEFAULT_SIGNIN_LABEL) %
        replacements
    )
    context['error_message'] = (
        error_message or
        DEFAULT_ERROR_MESSAGE % replacements
    )

    return template.loader.render_to_string(
        'twitter/login_button.html', context)


@register.simple_tag(takes_context=True)
def twitter_share(context, label=None, css_class=None, icon_class=None,
                  **kwargs):
    """
    Renders a 'Tweet' button to share content on Twitter.

    Parameters:
        - label: the text to show as button label.
        - css_class: the class to apply to the button

    Keyword arguments:
        - hashtags: Comma separated hashtags appended to tweet text.
        - related: Related accounts.
        - text: Default Tweet text.
        - url: URL of the page to share.
        - via: Screen name of the user to attribute the Tweet to.

    Please visit https://dev.twitter.com/docs/tweet-button for a detailed
    reference of the arguments supported by Twitter.
    """
    # Base javascript for the button.
    base_script = (
        "javascript:(function(){window.twttr=window.twttr||{};var D=550,"
        "A=450,C=screen.height,B=screen.width,H=Math.round((B/2)-(D/2)),"
        "G=0,F=document,E;if(C>A){G=Math.round((C/2)-(A/2))}"
        "window.twttr.shareWin=window.open('%(__URL__)s','','left='+H+',"
        "top='+G+',width='+D+',height='+A+',personalbar=0,toolbar=0,"
        "scrollbars=1,resizable=1');E=F.createElement('script');"
        "E.src=('https:'==document.location.protocol?'https://':'http://')+"
        "'platform.twitter.com/widgets.js';"
        "F.getElementsByTagName('head')[0].appendChild(E)}());"
    )

    # Preparing the url with the encoded parameters.
    r = requests.Request(
        url='https://twitter.com/share', params=kwargs).prepare()

    context['label'] = label or _('Tweet')
    context['css_class'] = css_class
    context['icon_class'] = icon_class
    context['script'] = base_script % {'__URL__': r.url}

    return template.loader.render_to_string(
        'twitter/share_button.html', context)



# -*- coding: utf-8 -*-
import httpretty
import json

from urlparse import parse_qs, urlparse, urlunparse

from django.core.urlresolvers import reverse
from django.test import Client, TestCase

from sure import expect

from socialnetworks.twitter.clients import TwitterClient
from socialnetworks.twitter.models import TwitterOAuthProfile


class TwitterLoginTestCase(TestCase):
    def setUp(self):
        self.client = Client()
        self.login_url = reverse('socialnetworks:twitter:login')
        self.callback_url = reverse('socialnetworks:twitter:callback')

    def register_normal_flow(self):
        httpretty.enable()
        httpretty.register_uri(
            httpretty.POST,
            TwitterClient.request_token_url,
            body=json.dumps({
                'oauth_token': 'TestTwitterOAuthRequestToken',
                'oauth_token_secret': 'TestTwitterOAuthRequestTokenSecret',
                'oauth_callback_confirmed': True
            })
        )

    def test_view_redirects_to_login_dialog(self):
        self.register_normal_flow()
        response = self.client.post(self.login_url, follow=False)

        expect(response.status_code) \
            .should.be.equal(302)

        expect(response) \
            .have.key('Location')

        parsed = urlparse(response['Location'])
        query = parse_qs(parsed.query)
        url = urlunparse((
            parsed.scheme, parsed.netloc, parsed.path, '', '', ''))

        expect(url) \
            .should.be.equal(TwitterClient.authorization_url)

        expect(query) \
            .should.be.equal({
                'oauth_token': ['TestTwitterOAuthRequestToken'],
            })

    def test_view_sets_oauth_parameters_in_session(self):
        self.register_normal_flow()
        self.client.post(self.login_url, follow=False)

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('oauth_request_token') \
            .being.equal('TestTwitterOAuthRequestToken')

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('oauth_request_token_secret') \
            .being.equal('TestTwitterOAuthRequestTokenSecret')

    def test_view_sets_default_next_parameter_in_session(self):
        self.register_normal_flow()
        self.client.post('/social/twitter/login/', follow=False)

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('next') \
            .being.equal('/')

    def test_view_sets_next_parameter_in_session(self):
        self.register_normal_flow()
        self.client.post(self.login_url, {'next': '/some-url/'}, follow=False)

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('next') \
            .being.equal('/some-url/')

    def test_view_sets_truthy_only_login_parameter_in_session(self):
        self.register_normal_flow()
        self.client.post(self.login_url, {'only_login': True}, follow=False)

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('only_login') \
            .being.equal('True')

    def test_view_sets_falsy_only_login_parameter_in_session(self):
        self.register_normal_flow()
        self.client.post(self.login_url, {'only_login': False}, follow=False)

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('only_login') \
            .being.equal('False')


class TwitterCallbackTestCase(TestCase):
    def setUp(self):
        self.client = Client()
        self.login_url = reverse('socialnetworks:twitter:login')
        self.callback_url = reverse('socialnetworks:twitter:callback')
        self.setup_url = reverse('socialnetworks:twitter:setup')

    def register_normal_flow(self):
        httpretty.enable()
        httpretty.register_uri(
            httpretty.POST,
            TwitterClient.request_token_url,
            body=json.dumps({
                'oauth_token': 'TestTwitterOAuthRequestToken',
                'oauth_token_secret': 'TestTwitterOAuthRequestTokenSecret',
                'oauth_callback_confirmed': True
            })
        )
        httpretty.register_uri(
            httpretty.POST,
            TwitterClient.access_token_url,
            body=json.dumps({
                'oauth_token': 'TestTwitterOAuthAccessToken',
                'oauth_token_secret': 'TestTwitterOAuthAccessTokenSecret',
            })
        )
        httpretty.register_uri(
            httpretty.GET,
            TwitterClient.token_debug_url,
            body=json.dumps({
                'id': 123456,
                'name': 'Test User',
                'screen_name': 'TestUserOfficial',
            })
        )

    def test_calling_without_verifier_returns_forbidden(self):
        response = self.client.get(self.callback_url)

        expect(response.status_code) \
            .should.be.equal(403)

    def test_calling_with_verifier_gets_access_token(self):
        callback_url = '{0}?oauth_token={1}&oauth_verifier={2}'

        self.register_normal_flow()
        self.client.post(self.login_url, follow=False)
        self.client.get(callback_url.format(
            self.callback_url,
            'TestTwitterOAuthRequestToken',
            'TestTwitterOAuthVerifier'
        ))

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('service_uid') \
            .to.being.equal('123456')

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('oauth_access_token') \
            .being.equal('TestTwitterOAuthAccessToken')

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('oauth_access_token_secret') \
            .being.equal('TestTwitterOAuthAccessTokenSecret')

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('oauth_request_token') \
            .being.equal('TestTwitterOAuthRequestToken')

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('oauth_request_token_secret') \
            .being.equal('TestTwitterOAuthRequestTokenSecret')

        expect(self.client.session) \
            .to.have.key('socialnetworks:twitter') \
            .to.have.key('oauth_refresh_token') \
            .being.none

    def test_calling_with_verifier_creates_new_profile(self):
        callback_url = '{0}?oauth_token={1}&oauth_verifier={2}'

        expect(TwitterOAuthProfile.objects.count()) \
            .should.be.equal(0)

        self.register_normal_flow()
        self.client.post(self.login_url, follow=False)

        response = self.client.get(callback_url.format(
            self.callback_url,
            'TestTwitterOAuthRequestToken',
            'TestTwitterOAuthVerifier'
        ))

        expect(TwitterOAuthProfile.objects.count()) \
            .should.be.equal(1)

        profile = TwitterOAuthProfile.objects.latest('id')

        expect(profile) \
            .to.have.property('user') \
            .being.none

        expect(profile) \
            .to.have.property('service_uid') \
            .being.equal('123456')

        expect(profile) \
            .to.have.property('oauth_access_token') \
            .being.equal('TestTwitterOAuthAccessToken')

        expect(profile) \
            .to.have.property('oauth_access_token_secret') \
            .being.equal('TestTwitterOAuthAccessTokenSecret')

        expect(profile) \
            .to.have.property('oauth_request_token') \
            .being.equal('TestTwitterOAuthRequestToken')

        expect(profile) \
            .to.have.property('oauth_request_token_secret') \
            .being.equal('TestTwitterOAuthRequestTokenSecret')

        expect(response.status_code) \
            .should.be.equal(302)

        expect(response) \
            .have.key('Location') \
            .being.equal('http://testserver' + self.setup_url)



from django.db import models
from django.utils.translation import ugettext_lazy as _

from ..core.models import BaseOAuth2Profile


class MovesAppOAuthProfile(BaseOAuth2Profile):
    """
    Model to store the Moves app OAuth related data.
    """
    class Meta:
        verbose_name = _('moves app profile')
        verbose_name_plural = _('moves app profiles')



from functools import wraps

from django.core import signing
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect

from .clients import TwitterClient
from .settings import COOKIE_MAX_AGE, SESSION_KEY, SESSION_FIELDS


def fetch_twitter_data(function):
    """
    Decorator that extends a view to allow it to fetch the user's
    data from Twitter.
    """
    @wraps(function)
    def function_wrapper(request, *args, **kwargs):
        # Check if the key for the Twitter session is in the user's session
        # and deletes it if it is present.
        if SESSION_KEY in request.session:
            del request.session[SESSION_KEY]

        # Tries to create a Twitter client for the current user.
        try:
            client = TwitterClient(request.user.twitteroauthprofile)
        except:
            client = None

        # If the client for this user was successfully created and the
        # user's data is not retrieved yet from Twitter: checks the
        # validity of the current access token, if the token is valid
        # retrieves the data from Twitter, if the token is invalid then
        # requests a new token.
        if client and SESSION_KEY not in request.COOKIES:
            if not client.debug_access_token():
                return HttpResponseRedirect(reverse(
                    'socialnetworks:twitter:login'))

            else:
                # Retrieves the data from Twitter.
                r = client.get('account/verify_credentials.json')
                data = dict(filter(
                    lambda i: i[0] in SESSION_FIELDS, r.items()))

                # Protects the user potentially sensible data.
                data = signing.dumps(data)

                # Stores the retrieved data temporarily in the user session
                # if required to access it in the view.
                request.session[SESSION_KEY] = data

                # Creates the response object and stores the retrieved data
                # in a signed cookie that its valid only for the seconds
                # specified in settings.
                response = function(request, *args, **kwargs)
                response.set_signed_cookie(
                    SESSION_KEY, value=data,
                    httponly=True, max_age=COOKIE_MAX_AGE
                )

                return response

        # If there is no client for the current user but it has data stored
        # in its session (when a user disconnects its profile), removes
        # the data from its cookies.
        elif not client and SESSION_KEY in request.COOKIES:
            # Stores temporarily a empty data for the session, this way
            # we can check quickly in the view if the user has disconnected
            # its profile.
            request.session[SESSION_KEY] = None

            # Creates the response object and deletes the cookie from it.
            response = function(request, *args, **kwargs)
            response.delete_cookie(SESSION_KEY)

            return response

        return function(request, *args, **kwargs)

    return function_wrapper



from django.core.exceptions import ImproperlyConfigured

from ..core.settings import CONFIGURATION, COOKIE_MAX_AGE


# Tries to get the PayPal configuration, if the configuration is not
# provided or is incorrect raises an ImproperlyConfigured exception.
PAYPAL = CONFIGURATION.get('PAYPAL', None)

if not PAYPAL:
    raise ImproperlyConfigured(
        'You have socialnetworks.paypal in your ISTALLED_APPS, but you do '
        'not specify any "PAYPAL" settings inside '
        'SOCIALNETWORKS_CONFIGURATION.'
    )

else:
    APP_ID = PAYPAL.get('APP_ID', None)
    APP_SECRET = PAYPAL.get('APP_SECRET', None)
    APP_ACCESS_TOKEN = PAYPAL.get('APP_ACCESS_TOKEN', None)
    SCOPE = ' '.join(PAYPAL.get('SCOPE', ['openid', 'email']))
    SESSION_KEY = PAYPAL.get('SESSION_KEY', 'dsnpp')
    SESSION_FIELDS = ','.join(PAYPAL.get('SESSION_FIELDS', []))
    IS_LIVE = PAYPAL.get('IS_LIVE', False)
    SETUP_URL_NAME = PAYPAL.get('SETUP_URL_NAME', None)

    if not APP_ID:
        raise ImproperlyConfigured(
            'A proper "APP_ID" must be specified in order to use '
            'socialnetworks.paypal module.'
        )

    if not APP_SECRET:
        raise ImproperlyConfigured(
            'A proper "APP_SECRET" must be specified in order to use '
            'socialnetworks.paypal module.'
        )



import json

from base64 import b64encode
from datetime import timedelta
from urlparse import urlparse, parse_qs

from django.contrib import messages
from django.contrib.auth import get_user_model
from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import resolve, reverse
from django.http import HttpResponseForbidden
from django.shortcuts import redirect
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _
from django.views.generic.base import View, TemplateView

from . import settings
from .utils import compose_username, from_timestamp, to_timestamp
from ..signals import activation, connect, disconnect, login


class OAuthMixin(object):
    """
    Mixin that defines the necessary methods for OAuth views.
    OAuth views must be inherited from this class.
    """
    # The client class that defines the necessary information to perform
    # the OAuth flow.
    client_class = None

    def __init__(self, *args, **kwargs):
        super(OAuthMixin, self).__init__(*args, **kwargs)

        if self.client_class is not None:
            self.client = self.client_class()

        else:
            raise ImproperlyConfigured(
                "A valid client_class must be specified to initialize a "
                "'%s' view." % self.__class__
            )

    def _build_absolute_uri(self, uri):
        """
        Compose and return the fully qualified url for the given uri by
        wrapping the build_abosulte_uri method of the current request..
        """
        return self.request.build_absolute_uri(uri)

    def _get_session(self):
        """
        Returns the current session's object to store the data that will
        be used trough the OAuth flow.
        """
        if self.client.session_key not in self.request.session:
            self.request.session[self.client.session_key] = {}

        return self.request.session[self.client.session_key]

    def _set_session(self, dictionary):
        """
        Updates the current session's object, useful if the backend is not
        configured to save session in every request.
        """
        self.request.session[self.client.session_key] = dictionary

    def _get_namespace(self):
        """
        Return the namespace of the current path to build uris without
        worrying about user assignaed url namespaces.
        """
        return resolve(self.request.path).namespace

    def _prepend_namespace(self, url_name):
        """
        Return the given url name prepended with the current namespace.
        Must be used only for build url names that take care about namespace.
        """
        return ':'.join((self._get_namespace(), url_name))

    def session_get(self, key):
        """
        Returns the provided key value from the current session's object. If
        key is not in the object returns None.
        """
        return self._get_session().get(key, None)

    def session_put(self, **dictionary):
        """
        Puts the given dictionary into the current session's object.
        """
        session = self._get_session()
        session.update(**dictionary)
        self._set_session(session)

    def session_pop(self, key):
        """
        Returns the provided key value from the current session's object and
        removes the key from the object. If key is not in the object returns
        None.
        """
        session = self._get_session()
        value = session.pop(key, None)
        self._set_session(session)

        return value

    def session_clear(self):
        """
        Removes the object where the flow information of the service are
        stored from the current user's session.
        """
        if self.client.session_key in self.request.session:
            del self.request.session[self.client.session_key]

    def get_callback_url(self):
        """
        Return the callback url for the current request and service.
        """
        url_name = self._prepend_namespace('callback')
        url = reverse(url_name)

        return self._build_absolute_uri(url)

    def get_redirect_url(self):
        """
        Returns the url where the user will be redirected after its request
        is processed and is successfully logged in.
        """
        raise NotImplementedError

    def get_profile(self):
        """
        Returns the current user's OAuth profile or None.
        """
        raise NotImplementedError

    def create_new_user(self, data):
        """
        Returns a new user, created from the data dictionary.
        """
        raise NotImplementedError


class OAuthDialogRedirectView(OAuthMixin, View):
    """
    Base view that handles the redirection to the service
    authorization dialog.
    """
    def post(self, request, *args, **kwargs):
        # Clears the current session to avoid conflicts.
        self.session_clear()

        # Fetches the url where the user will be redirected when the
        # flow ends and stores it in the user's session.
        if 'next' in request.POST:
            self.session_put(**{'next': request.POST['next']})

        else:
            self.session_put(**{'next': '/'})

        if 'only_login' in request.POST:
            self.session_put(**{'only_login': request.POST['only_login']})

        if self.client.oauth_version == 1:
            # Gets the OAuth request token.
            credentials = self.client.get_request_token(
                callback=self.get_callback_url())

            request_token = credentials.get(self.client.request_token_label)
            request_token_secret = credentials.get(
                self.client.request_token_secret_label)

            # Appends the response to the user session.
            self.session_put(**{
                'oauth_request_token': request_token,
                'oauth_request_token_secret': request_token_secret
            })

        # Redirects the user to the authorization dialog.
        return redirect(self.get_redirect_url())

    def get_redirect_url(self):
        """
        Returns the url where the user should be redirected to request
        autorization to its account details.
        """
        if self.client.oauth_version == 1:
            param = self.session_get('oauth_request_token')

        elif self.client.oauth_version == 2:
            param = self.get_callback_url()

        return self.client.compose_authorization_url(param)


class OAuthCallbackView(OAuthMixin, View):
    """
    Base view that handles the callback redirection from the service.
    """
    def get(self, request, *args, **kwargs):
        # Protects the view to be accessed by non OAuth requests.
        if self.client.verifier_label not in request.GET:
            return HttpResponseForbidden()

        request_token = self.session_get('oauth_request_token')
        request_token_secret = self.session_get('oauth_request_token_secret')

        # Gets the OAuth access token.
        if self.client.oauth_version == 1:
            credentials = self.client.get_access_token(
                request_token=request_token,
                request_token_secret=request_token_secret,
                verifier=request.GET[self.client.verifier_label]
            )

        elif self.client.oauth_version == 2:
            credentials = self.client.get_access_token(
                verifier=request.GET[self.client.verifier_label],
                callback=self.get_callback_url(),
            )

        # Verifies if the service's user id was provided in the credentials,
        # if it was not provided then fetches it by debugging the access token.
        service_uid = credentials.get(self.client.uid_label, None)
        access_token = credentials.get(self.client.access_token_label, None)
        refresh_token = credentials.get(self.client.refresh_token_label, None)
        token_expiration = credentials.get(self.client.expiration_label, None)
        access_token_secret = credentials.get(
            self.client.access_token_secret_label, None)

        # Calculates the token expiration date.
        if token_expiration is not None:
            token_expiration = (
                timezone.now() + timedelta(seconds=int(token_expiration)))

        # Debugs the token.
        if service_uid is None:
            is_valid, debugged_data = self.client.debug_access_token(
                access_token)

            if is_valid:
                service_uid = debugged_data.get(self.client.uid_label)

        # Appends the response to the user session.
        profile_data = {
            'service_uid': service_uid,
            'oauth_access_token': access_token,
            'oauth_access_token_secret': access_token_secret,
            'oauth_access_token_expires_at': to_timestamp(token_expiration),
            'oauth_request_token': request_token,
            'oauth_request_token_secret': request_token_secret,
            'oauth_refresh_token': refresh_token
        }

        self.session_put(**profile_data)

        # Gets the lookups to retrieve or create the profile.
        lookup_kwargs = {'service_uid': service_uid}

        # Tries to retrieve the profile.
        try:
            profile = self.client.model.objects.get(**lookup_kwargs)
            created = False

        except self.client.model.DoesNotExist:
            profile = None

        # If there is no profile and the button clicked was only login
        # redirects the user to the page where it comes from, otherwise
        # creates a new profile with the given data.
        if not profile:
            if self.session_get('only_login'):
                parsed = urlparse(self.session_pop('only_login'))
                params = parse_qs(parsed.query)
                params.update(oauth_error=True)

                return redirect(self.client.encode_url(parsed.path, params))

            else:
                profile = self.client.model.objects.create(**lookup_kwargs)
                created = True

        # Reconvert the token expiration to a Python datetime object.
        profile_data.update(oauth_access_token_expires_at=from_timestamp(
            profile_data['oauth_access_token_expires_at']))

        # Updates the profile to make sure that we have always the most
        # recent token.
        profile.__dict__.update(**profile_data)
        profile.save()

        # If the profile was created or has no user then tries to attach the
        # profile to current user, if the user is not logged in then redirects
        # it to the final setup view to create a new user instance prefilled
        # with the data retrieved from the service's API.
        if created or not profile.user:
            if request.user.is_authenticated():
                profile.user = request.user
                profile.save()

                # Tells to the site that the user has connected its profile.
                connect.send(
                    sender=self.__class__, user=request.user,
                    service=self.client.service_name.lower()
                )

                # Tells to the user that his connection was successful.
                tags = 'social %s' % self.client.service_name.lower()
                messages.success(request, _(
                    'Your %(service)s profile was successfully connected '
                    'with your user account.'
                ) % {'service': self.client.service_name}, extra_tags=tags)

            else:
                self.session_put(**{'new_user': True})

        elif (profile.user and profile.user != request.user and
                request.user.is_authenticated()):

            # Tells to the user that his connection was unsuccessful.
            tags = 'social %s' % self.client.service_name.lower()
            messages.error(request, _(
                'This %(service)s profile is already connected with another '
                'user account.'
            ) % {'service': self.client.service_name}, extra_tags=tags)

        else:
            if not request.user.is_authenticated():
                # Logs the user in if it is not logged in yet.
                self.client.login(request, self.session_get('service_uid'))

                # Tells to the site that the user was logged in.
                login.send(
                    sender=self.__class__, user=request.user,
                    service=self.client.service_name.lower()
                )

            # Tells to the site that the user has connected its profile.
            connect.send(
                sender=self.__class__, user=request.user,
                service=self.client.service_name.lower()
            )

        return redirect(self.get_redirect_url())

    def get_redirect_url(self):
        """
        Return the url to redirect the user after processing the
        current request.
        """
        if self.session_get('new_user') is not None:
            url_name = self._prepend_namespace('setup')
            url = reverse(url_name)

        else:
            url = self.session_pop('next') or '/'

        return self._build_absolute_uri(url)


class OAuthSetupView(OAuthMixin, TemplateView):
    """
    Base View that handles the setup for the account after the access
    token is provided.
    """
    template_name = settings.SETUP_TEMPLATE
    setup_url = None

    def get_profile(self):
        """
        Tries to return the OAuth profile by retrieving the service_uid
        in the current session.
        """
        try:
            return self.client.model.objects.get(
                service_uid=self.session_get('service_uid'))

        except self.client.model.DoesNotExist:
            return None

    def create_new_user(self, data):
        """
        Create and return a new user with the given data.
        """
        UserModel = get_user_model()

        # Creates the new user.
        user = UserModel.objects.create_user(
            data['username'], email=data['email']
        )

        # Updates the user data.
        user.first_name = data['first_name']
        user.last_name = data['last_name']
        user.save()

        return user

    def retrieve_user_data(self):
        """
        Retrieve the available user data from the service's API.
        """
        profile = self.get_profile()
        profiled_client = self.client_class(profile)

        return profiled_client.retrieve_user_data()

    def get_context_data(self, **kwargs):
        context = super(OAuthSetupView, self).get_context_data(**kwargs)
        context['service'] = self.client.service_name
        context['form'] = kwargs.get('form', settings.SETUP_FORM_CLASS(kwargs))

        return context

    def get(self, request, *args, **kwargs):
        # Protects the view to be accessed by non OAuth requests.
        if (request.user.is_authenticated() or
                not self.session_get('service_uid')):

            return HttpResponseForbidden()

        # Fetches the current OAuth profile.
        profile = self.get_profile()

        # If the profile has no user tries to attach the current user to it,
        # if the user is not logged in then creates a new one.
        if profile.user is None:
            # Fetches the user data from the service.
            UserModel = get_user_model()
            user_data = self.retrieve_user_data()
            check = lambda k: k in user_data and user_data[k]

            if settings.EMAIL_IS_USERNAME and check('email'):
                user_data.update(username=user_data['email'])

            # Check if the retrieved data is enough to create a new user or
            # fetch an existing user instance from the database.
            # If a existent user matches the data then links the OAuth profile
            # to its account.
            if check('email'):
                filters = {'email__iexact': user_data['email']}

                try:
                    user = UserModel.objects.get(**filters)
                    created = False

                except UserModel.DoesNotExist:
                    user = None
                    created = False

                if user is None and check('username'):
                    filters = {'username__iexact': user_data['username']}

                    if not UserModel.objects.filter(**filters):
                        user = self.create_new_user(user_data)
                        created = True

                if user is not None:
                    # Attaches the user to the profile.
                    profile.user = user
                    profile.save()

                    service = self.client.service_name.lower()

                    # Tells to the site that the user has linked its profile.
                    connect.send(sender=self.__class__, user=user,
                                 service=service)

                    # If the user was retrieved from the database and it is
                    # not active and ```ACTIVATE_ALREADY_REGISTERED_USERS```
                    # is True in settings then activates the user account and
                    # tell the site that the user was activated.
                    if (not created and not user.is_active and
                            settings.ACTIVATE_ALREADY_REGISTERED_USERS):

                        user.is_active = True
                        user.save()

                        activation.send(
                            sender=self.__class__, user=user,
                            service=service
                        )

                    # Authenticates the new user.
                    self.client.login(request, self.session_get('service_uid'))

                    # Tells to the site that the user was logged in.
                    login.send(sender=self.__class__, user=user,
                               service=service)

                    # Redirects the user to the proper url.
                    return redirect(self.get_redirect_url())

            # If no user was matched or created then redirect the user to the
            # final sertup view.
            # Suggest an username that does not exists in the site.
            user_data['username'] = compose_username(user_data)

            # Redirect the user to a custom setup url if provided.
            if self.setup_url is not None:
                url = reverse(self.setup_url)
                response = redirect(self._build_absolute_uri(url))

                # Set a temporal cookie with a max age of 2 minutes with the
                # user retrieved data to facilitate access for views that does
                # not depende on django rendering. The data is dumped as JSON
                # and then base64 endoced to can be correctly stored in the
                # cookie without braking it.
                encoded_data = b64encode(json.dumps(user_data))
                response.set_cookie('dsnstp', encoded_data, max_age=120)

                return response

            # Otherwise render the form to setup the user data.
            context = self.get_context_data(**user_data)

            return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        # Protects the view to be accessed by non OAuth requests.
        if (request.user.is_authenticated() or
                not self.session_get('service_uid')):

            return HttpResponseForbidden()

        # Fetches the current OAuth profile.
        profile = self.get_profile()
        form = settings.SETUP_FORM_CLASS(request.POST)

        if form.is_valid():
            # Composes the data for the user creation.
            user_data = {
                'email': form.cleaned_data['email'],
                'first_name': form.cleaned_data['first_name'],
                'last_name': form.cleaned_data['last_name']
            }

            # Sets the user's username.
            if settings.EMAIL_IS_USERNAME:
                user_data.update({'username': user_data['email']})
            else:
                user_data.update({'username': form.cleaned_data['username']})

            # Gets a new user and attaches it to the profile.
            user = self.create_new_user(user_data)
            profile.user = user
            profile.save()

            # Tells to the site that the user has connected its profile.
            connect.send(
                sender=self.__class__, user=user,
                service=self.client.service_name.lower()
            )

            # Authenticates the new user.
            self.client.login(request, self.session_get('service_uid'))

            # Tells to the site that the user was logged in.
            login.send(
                sender=self.__class__, user=user,
                service=self.client.service_name.lower()
            )

            # Redirects the user to the proper url.
            response = redirect(self.get_redirect_url())
            response.delete_cookie('dsnstp')

            return response

        else:
            if self.setup_url is not None:
                url = reverse(self.setup_url)
                response = redirect(self._build_absolute_uri(url))

                # Set a temporal cookie with a max age of 2 minutes with the
                # user retrieved data to facilitate access for views that does
                # not depende on django rendering. The data is dumped as JSON
                # and then base64 endoced to can be correctly stored in the
                # cookie without braking it.
                encoded_data = b64encode(json.dumps(form.data))
                response.set_cookie('dsnstp', encoded_data, max_age=120)

                return response

            # Otherwise render the form to setup the user data.
            context = self.get_context_data(**{'form': form})

            return self.render_to_response(context)

    def get_redirect_url(self):
        """
        Return the url to redirect the user when the setup of the new account
        has successfully finished.
        """
        url = self.session_pop('next') or '/'

        return self._build_absolute_uri(url)


class OAuthDisconnectView(OAuthMixin, View):
    """
    Base view that handles the flow to disconnect a service account from a
    site account.
    """
    def get_profile(self):
        # Tries to return the OAuth profile by retrieving the current
        # user's id.
        try:
            return self.client.model.objects.get(
                user__id=self.request.user.id)

        except self.client.model.DoesNotExist:
            return None

    def post(self, request, *args, **kwargs):
        # Fetches the OAuth profile.
        profile = self.get_profile()

        # If the profile is given then deletes it.
        if profile:
            user = profile.user
            profile.delete()

            # Tells to the site that the profile was disconnected.
            disconnect.send(
                sender=self.__class__, user=user,
                service=self.client.service_name.lower()
            )

            # Tells to the user that the disconnection was successful.
            tags = 'social %s' % self.client.service_name.lower()
            messages.success(request, _(
                'Your %(service)s profile was successfully disconnected from '
                'your user account.'
            ) % {'service': self.client.service_name}, extra_tags=tags)

        return redirect(request.POST.get('next', '/'))



from . import settings
from .models import FacebookOAuthProfile
from ..core.clients import OAuth2Client


class FacebookClient(OAuth2Client):
    """
    Client to connect to the Facebook graph API.
    """
    service_name = 'Facebook'
    app_key = settings.APP_ID
    app_secret = settings.APP_SECRET
    app_access_token = settings.APP_ACCESS_TOKEN
    scope = settings.SCOPE
    model = FacebookOAuthProfile

    expiration_label = 'expires'
    authorization_url = 'https://www.facebook.com/v2.2/dialog/oauth'
    access_token_url = 'https://graph.facebook.com/v2.2/oauth/access_token'
    token_debug_url = 'https://graph.facebook.com/v2.2/debug_token'
    service_api_url = 'https://graph.facebook.com/v2.2/'
    session_key = 'socialnetworks:facebook'

    def compose_authorization_url(self, callback_url):
        """
        Return the url to request user authorization at Facebook.
        """
        params = {
            'client_id': self.app_key,
            'redirect_uri': callback_url,
            'scope': self.scope,
        }

        return self.encode_url(self.authorization_url, params)

    def get_app_access_token(self):
        """
        Returns the app access token if it is defined in settings,
        otherwise fetches the token from Facebook.
        """
        if self.app_access_token:
            return self.app_access_token

        else:
            params = {
                'client_id': self.app_key,
                'client_secret': self.app_secret,
                'grant_type': 'client_credentials'
            }

            r = self._get(self.access_token_url, params=params)

            return self.parse_response(r.content)['access_token']

    def debug_access_token(self, token=None):
        """
        Check if the given access token is yet valid.

        Return a tuple of two objects where the first element is a boolean
        that tells whether the token is valid or not and the second element is
        the data resulting of the token validation.
        """
        if token is None:
            token = self._oauth_data['access_token']

        params = {
            'input_token': token,
            'access_token': self.get_app_access_token()
        }

        r = self._get(self.token_debug_url, params=params)
        data = r.json()

        return (data['data']['is_valid'], data['data'])

    def retrieve_user_data(self):
        """
        Return the available data of the user from Facebook.
        """
        # Defines the fields to retrieve.
        fields = ','.join(['first_name', 'last_name', 'email'])

        return self.get('me', params={'fields': fields})
